@@ -1,17 +1,19 @@
 import hashlib
 from pathlib import Path
-from typing import Dict, List, Optional, Union
+from typing import TYPE_CHECKING, Dict, List, Optional, Union
 
 import numpy as np
-from docarray import DocumentArray
-from docarray.math.distance import cdist
-from docarray.math.helper import top_k
 from loguru import logger
 
+if TYPE_CHECKING:
+    from docarray import DocumentArray
+
 from .container import CellContainer
 from .core import PQCodec, VQCodec
 from .enums import Metric
 from .filter import Filter
+from .helper import setup_logging
+from .math import cdist, top_k
 
 
 class PQLite(CellContainer):
@@ -21,7 +23,7 @@ class PQLite(CellContainer):
 
         .. highlight:: python
         .. code-block:: python
-            pqlite = PQLite(dim=256, metric=pqlite.Metric.EUCLIDEAN)
+            pqlite = PQLite(dim=256, metric='cosine')
 
     :param dim: dimensionality of input vectors. there are 2 constraints on dim:
             (1) it needs to be divisible by n_subvectors; (2) it needs to be a multiple of 4.*
@@ -35,8 +37,9 @@ class PQLite(CellContainer):
             if any cell has reached its capacity, that cell will be automatically expanded.
             If you need to add vectors frequently, a larger value for init_size is recommended.
     :param data_path: location of directory to store the database.
-    :param create: if False, do not create the directory path if it is missing.
+    :param create_if_missing: if False, do not create the directory path if it is missing.
     :param read_only: if True, the index is not writable.
+    :param verbose: if True, will print the debug logging info.
 
     .. note::
         Remember that the shape of any tensor that contains data points has to be `[n_data, dim]`.
@@ -45,7 +48,7 @@ class PQLite(CellContainer):
     def __init__(
         self,
         dim: int,
-        metric: Union[str, Metric] = Metric.COSINE,
+        metric: Union[str, Metric] = 'cosine',
         n_cells: int = 1,
         n_subvectors: Optional[int] = None,
         n_probe: int = 16,
@@ -53,17 +56,21 @@ class PQLite(CellContainer):
         expand_step_size: int = 10240,
         columns: Optional[List[tuple]] = None,
         data_path: Union[Path, str] = Path('./data'),
-        create: bool = True,
+        create_if_missing: bool = True,
         read_only: bool = False,
+        verbose: bool = False,
         *args,
         **kwargs,
     ):
+        setup_logging(verbose)
+
         if n_subvectors:
             assert (
                 dim % n_subvectors == 0
             ), '"dim" needs to be divisible by "n_subvectors"'
 
         self.n_subvectors = n_subvectors
+        ###self.n_probe = max(n_probe, n_cells)
         self.n_probe = max(n_probe, n_cells)
         self.n_cells = n_cells
 
@@ -76,7 +83,7 @@ class PQLite(CellContainer):
         self.read_only = read_only
 
         data_path = Path(data_path)
-        if create:
+        if create_if_missing:
             data_path.mkdir(parents=True, exist_ok=True)
         self.data_path = data_path
 
@@ -117,13 +124,15 @@ class PQLite(CellContainer):
         if self.total_docs > 0:
             self._rebuild_index()
 
-    def _sanity_check(self, x: np.ndarray):
+    def _sanity_check(self, x: 'np.ndarray'):
         assert len(x.shape) == 2
         assert x.shape[1] == self.dim
 
         return x.shape
 
-    def train(self, x: np.ndarray, auto_save: bool = True, force_retrain: bool = False):
+    def train(
+        self, x: 'np.ndarray', auto_save: bool = True, force_retrain: bool = False
+    ):
         """Train pqlite with training data.
 
         :param x: the ndarray data for training.
@@ -156,7 +165,40 @@ class PQLite(CellContainer):
         if auto_save:
             self.dump_model()
 
-    def index(self, docs: DocumentArray, **kwargs):
+    def partial_train(
+        self, x: np.ndarray, auto_save: bool = True, force_retrain: bool = False
+    ):
+        """Train vector quantizers and product quantizers with a minibatch of  data.
+
+        :param x: the ndarray data for training.
+        :param auto_save: if False, will not dump the trained model to ``model_path``.
+        :param force_retrain: if True, enforce to retrain the model, and overwrite the model if ``auto_save=True``.
+
+        """
+        n_data, _ = self._sanity_check(x)
+
+        if self.vq_codec:
+            logger.info(
+                f'Partial training VQ codec (K={self.n_cells}) with {n_data} data...'
+            )
+            self.vq_codec.partial_fit(x)
+
+        if self.pq_codec:
+            logger.info(
+                f'Partial training PQ codec (n_subvectors={self.n_subvectors}) with {n_data} data...'
+            )
+            self.pq_codec.partial_fit(x)
+
+    def build_codebook(self):
+        """Constructs a codebooks for the vq_codec and pq_codec.
+        This step is not necessary if full KMeans is trained used calling `.fit`.
+        """
+        if self.vq_codec:
+            self.vq_codec.build_codebook()
+        if self.pq_codec:
+            self.pq_codec.build_codebook()
+
+    def index(self, docs: 'DocumentArray', **kwargs):
         """Index new documents
 
         :param docs: the documents to index
@@ -175,10 +217,9 @@ class PQLite(CellContainer):
             if self.vq_codec
             else np.zeros(n_data, dtype=np.int64)
         )
-
         return super(PQLite, self).insert(x, assigned_cells, docs)
 
-    def update(self, docs: DocumentArray, **kwargs):
+    def update(self, docs: 'DocumentArray', **kwargs):
         """Update existing documents
 
         :param docs: the documents to update
@@ -200,7 +241,7 @@ class PQLite(CellContainer):
 
     def search(
         self,
-        docs: DocumentArray,
+        docs: 'DocumentArray',
         filter: Dict = {},
         limit: int = 10,
         include_metadata: bool = True,
@@ -224,7 +265,7 @@ class PQLite(CellContainer):
 
     def _search_documents(
         self,
-        query_np,
+        query_np: 'np.ndarray',
         filter: Dict = {},
         limit: int = 10,
         include_metadata: bool = True,
@@ -243,10 +284,11 @@ class PQLite(CellContainer):
         )
         return match_dists, match_docs
 
+        return dists, ids
+
     def _cell_selection(self, query_np, limit):
 
         n_data, _ = self._sanity_check(query_np)
-        assert 0 < limit <= 1024
 
         if self.vq_codec:
             dists = cdist(
@@ -270,15 +312,13 @@ class PQLite(CellContainer):
         #     n_probe_list = torch.ceil(normalized_entropy * max_n_probe).long()
         # else:
         #     n_probe_list = None
-
         return cells
 
     def search_numpy(
         self,
-        query_np: np.ndarray,
+        query_np: 'np.ndarray',
         filter: Dict = {},
         limit: int = 10,
-        include_metadata: bool = True,
         **kwargs,
     ):
         """Search the index and return distances to the query and ids of the closest documents.
@@ -286,13 +326,12 @@ class PQLite(CellContainer):
         :param query_np: matrix containing query vectors as rows
         :param filter: the filtering conditions
         :param limit: the number of results to get for each query document in search
-        :param include_metadata: whether to return document metadata in response.
         """
 
         dists, doc_ids = self._search_numpy(query_np, filter, limit)
         return dists, doc_ids
 
-    def _search_numpy(self, query_np, filter: Dict = {}, limit: int = 10):
+    def _search_numpy(self, query_np: 'np.ndarray', filter: Dict = {}, limit: int = 10):
         """Search approximate nearest vectors in different cells, returns distances and ids
 
         :param query_np: matrix containing query vectors as rows
@@ -311,18 +350,17 @@ class PQLite(CellContainer):
         )
         return dists, ids
 
-    def delete(self, docs: Union[DocumentArray, List[str]]):
+    def delete(self, docs: Union['DocumentArray', List[str]]):
         """Delete entries from the index by id
 
         :param docs: the documents to delete
         """
-        doc_ids = docs.get_attributes('id') if isinstance(docs, DocumentArray) else docs
-        super().delete(doc_ids)
+        super().delete(docs if isinstance(docs, list) else docs.get_attributes('id'))
 
     def clear(self):
         """Clear the whole database"""
         for cell_id in range(self.n_cells):
-            logger.info(f'Clear the index of cell-{cell_id}')
+            logger.debug(f'Clear the index of cell-{cell_id}')
             self.vec_index(cell_id).reset()
             self.cell_table(cell_id).clear()
             self.doc_store(cell_id).clear()
@@ -332,12 +370,12 @@ class PQLite(CellContainer):
         for cell_id in range(self.n_cells):
             self.doc_store(cell_id).close()
 
-    def encode(self, x: np.ndarray):
+    def encode(self, x: 'np.ndarray'):
         n_data, _ = self._sanity_check(x)
         y = self.pq_codec.encode(x)
         return y
 
-    def decode(self, x):
+    def decode(self, x: 'np.ndarray'):
         assert len(x.shape) == 2
         assert x.shape[1] == self.n_subvectors
         return self.pq_codec.decode(x)
